#!/bin/bash

# SBC4 image version
#  - 1.0.0: initial (4.9.35 kernel)
#  - 1.0.1: uboot upgraded, better HDMI, USB support
#  - 1.1.0: kernel upgraded to 4.14.24, fixes some USB issues
#  - 1.2.0: uboot upgraded, fixes issue with one 1 cpu core being detected
#  - 1.2.1: uboot upgraded to 2018.05, dram slowed to 384MHz
#  - 2.0.0: Debian11, 5.10.60 kernel, 2018.05 uboot - upgrade via SD card flash
VERSION="1.2.1"

UPGRADE_KERNEL=false
UPGRADE_UBOOT=false

KERNELVER="4.14.24"
KERNELDEB="http://www.phidgets.com/downloads/phidgetsbc/SBC3003/kernel/linux-image-next-sunxi_5.41_armhf.deb"
UBOOTDEB="https://www.phidgets.com/downloads/phidgetsbc/SBC3003/kernel/linux-u-boot-next-phidgetsbc4_5.59_armhf.deb"
VINTHUBPKG="vinthub-${KERNELVER}-sunxi"

LOG=${0}.log

RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m' # No Color

CURRENT_VERSION=$(cat /etc/phidgetsbc_version | grep "^version" | awk '{ print ($2) }')

detect_version() {
	# Check which release we're on
	# 1.0.0 requires a uBoot and kernel upgrade
	# 1.0.1 requires a uBoot and kernel upgrade
	# 1.1.0 requires just a uBoot upgrade
	# 1.2.0 requires just a uBoot upgrade

	if [ "$CURRENT_VERSION" != "2.0.0" ] ; then
		printf "${RED}NOTE:${NC} Version 2.0 is available, but must be installed by re-flashing the SD card.\n"
		printf "      See: https://www.phidgets.com/docs/OS_-_Phidget_SBC#Writing_Image_to_SD_Card\n\n"
	fi

	if [ "$CURRENT_VERSION" == "1.0.0" ] ; then
		printf "Detected Version $CURRENT_VERSION - Upgrading Bootloader and Kernel\n"
		UPGRADE_KERNEL=true
		UPGRADE_UBOOT=true
	elif [ "$CURRENT_VERSION" == "1.0.1" ] ; then
		printf "Detected Version $CURRENT_VERSION - Upgrading Bootloader and Kernel\n"
		UPGRADE_KERNEL=true
		UPGRADE_UBOOT=true
	elif [ "$CURRENT_VERSION" == "1.1.0" ] ; then
		printf "Detected Version $CURRENT_VERSION - Upgrading Bootloader\n"
		UPGRADE_UBOOT=true
	elif [ "$CURRENT_VERSION" == "1.2.0" ] ; then
		printf "Detected Version $CURRENT_VERSION - Upgrading Bootloader\n"
		UPGRADE_UBOOT=true
	elif [ "$CURRENT_VERSION" == "1.2.1" ] ; then
		printf "Detected Version $CURRENT_VERSION - No upgrades required\n"
		exit 0
	elif [ "$CURRENT_VERSION" == "2.0.0" ] ; then
		printf "Detected Version $CURRENT_VERSION - No upgrades required\n"
		exit 0
	else
		printf "Detected Unexpected Version [${CURRENT_VERSION}] - Cannot continue\n"
		return 1
	fi

	return 0
}

upgrade_kernel() {

	printf "Downloading new kernel...\n"
	wget --no-check-certificate ${KERNELDEB} -q --show-progress -O files/$(basename ${KERNELDEB}) || return 1
	printf "done.\n"

	printf "Installing new kernel... "
	dpkg --install files/$(basename ${KERNELDEB}) >>${LOG} 2>&1 || return 1
	printf "done.\n"

	printf "Installing new VINT Hub driver... "
	apt-get -y install --reinstall ${VINTHUBPKG} >>${LOG} 2>&1 || return 1
	printf "done.\n"

	depmod >>${LOG} 2>&1 || return 1

	return 0
}

upgrade_uboot() {

	printf "Downloading new bootloader...\n"
	wget --no-check-certificate ${UBOOTDEB} -q --show-progress -O files/$(basename ${UBOOTDEB}) || return 1
	printf "done.\n"

	cp files/boot.bmp /boot/ >>${LOG} 2>&1 || return 1
	cp files/boot.cmd /boot/ >>${LOG} 2>&1 || return 1
	cp files/boot.scr /boot/ >>${LOG} 2>&1 || return 1
	cp files/ubootEnv.txt /boot/ >>${LOG} 2>&1 || return 1
	cp files/sun7i-a20-phidget-sbc4.dtb /boot/ >>${LOG} 2>&1 || return 1

	apt-get remove linux-u-boot-phidgetsbc4-default >>${LOG} 2>&1 || true

	printf "Installing new bootloader... "
	dpkg --install files/$(basename ${UBOOTDEB}) >>${LOG} 2>&1 || return 1
	printf "done.\n"

	return 0
}

dowork() {
	printf "\nPerforming SBC3003 Upgrade\n\n"

	detect_version || return 1

	printf "Updating Packages... "
	apt-get update >>${LOG} 2>&1
	if [ $? -ne 0 ] ; then
		printf "${RED}fail${NC}\n"
		printf "Make sure SBC is connected to the internet and try again"
		return 1
	fi
	printf "done.\n"

	if [ "$UPGRADE_KERNEL" == "true" ] ; then
		upgrade_kernel
		if [ $? -ne 0 ] ; then
			printf "${RED}fail${NC}\n"
			return 1
		fi
	fi

	if [ "$UPGRADE_UBOOT" == "true" ] ; then
		upgrade_uboot
		if [ $? -ne 0 ] ; then
			printf "${RED}fail${NC}\n"
			return 1
		fi
	fi

	# Update the version string
	sed -i.bak "s/^version $CURRENT_VERSION/version $VERSION/g" /etc/phidgetsbc_version && {
		rm /etc/phidgetsbc_version.bak
	} || {
		printf "Error updating version file\n"
		return 1
	}

	return 0
}

dowork
result=$?

if [ $result -eq 0 ] ; then
	printf "\n${GREEN}SUCCESS${NC}\n\n"
	printf "Rebooting in 5 seconds...\n\n"
	sleep 5
	reboot
else
	printf "\n${RED}FAILED${NC}\nSee ${LOG} for details\n\n"
fi

exit $result
